**************************************************************
* SCROLL.ASM
**************************************************************
* CSG Applications Engineering
* Motorola SPS
*
* Software written to demonstrate scrolling long text messages
* across an LCD display.
*
* This is written for the MC68HC705L16, which provides built-in
* LCD drive capabilities.
* The LCD used is a Planar-Standish Model 4228 Multiplex
* 15-segment, 8-digit panel. (1/4 duty, 1/3 bias)
* An external interrupt provided by a button switch on IRQ1'
* selects the message to be displayed.
*
* Although this software was written for the 705L16 interface
* to raw LCD glass, it can be easily modified for use with a
* smart LCD module and a serial interface with another MCU.
*
*-------------------------------------------------------------
$BASE      10T                   ;Default assembler number base

*-------------------------------------------------------------
* Memory Equates
*-------------------------------------------------------------
RAMSPACE   EQU   $0040      ;Start of user RAM
ROMSPACE   EQU   $1000      ;Start of user ROM
RESETVEC   EQU   $FFFE      ;Reset vector
IRQVEC     EQU   $FFFA      ;IRQ' vector

*-------------------------------------------------------------
* Register Equates
*-------------------------------------------------------------
* Registers
MISC       EQU   $3E        ;Miscellaneous register
TBCR1      EQU   $10        ;Time base control register 1
LCDCR      EQU   $20        ;LCD control register
LCDDR      EQU   $21        ;First LCD data register location
INTCR      EQU   $08        ;Interrupt control register
INTSR      EQU   $09        ;Interrupt status register

* Bit locations
LCDE       EQU   $07        ;LCD enable bit in LCDCR
SYS0       EQU   $02        ;SYS0 bit in MISC
SYS1       EQU   $03        ;SYS1 bit in MISC
IRQ1E      EQU   $07        ;IRQ1 enable bit in INTCR
IRQ1S      EQU   $03        ;IRQ1 sensitivity bit
IRQ1F      EQU   $07        ;IRQ1 flag bit in INTSR
RIRQ1      EQU   $03        ;Reset IRQ1 flag bit

*-------------------------------------------------------------
* LCD Equates
*-------------------------------------------------------------
MAXCHARS   EQU   $08        ;Maximum characters per line of LCD
EOT        EQU   $04        ;End of string marker (ASCII EOT)

*-------------------------------------------------------------
* RAM Variables
*-------------------------------------------------------------
           ORG   RAMSPACE   ;Start of user RAM
TempX      RMB   1          ;Temporary register storage
TempA      RMB   1          ;Temporary register storage
TempData   RMB   1          ;Temp storage for LCD segment data
LCDReg     RMB   1          ;8-bit address pointer
Count      RMB   1          ;Counter variable
MsgIndex   RMB   1          ;Index counter variable
MsgStart   RMB   1          ;Stores starting point of string

*-------------------------------------------------------------
* Start of program code
*-------------------------------------------------------------
           ORG   ROMSPACE    ;Start of user EPROM
Start      BCLR  SYS0,MISC   ;Setup for f_op = f_osc/2
           BCLR  SYS1,MISC
           LDA   #$20        ;XOSC for time base
           STA   TBCR1       ;LCD clock = XOSC/128 = 256Hz
           BSET  LCDE,LCDCR  ;Enable LCD
           BSET  IRQ1S,INTCR ;Set edge-level sensitivity
           BSET  IRQ1E,INTCR ;Enable IRQ1 interrupts
           BSET  RIRQ1,INTSR ;Clear IRQ1 flag
           CLI               ;Enable interrupts

*-------------------------------------------------------------
* Initialize string to be initially displayed.
* When a new message is desired, the same LDA offset, JSR LoadMsg
* steps should be followed.
*-------------------------------------------------------------
           LDA   #Msg1      ;Load offset of desired string
           JSR   LoadMsg    ;Initialize message variables

*-------------------------------------------------------------
* Main loop
* UpdateLCD might be one of many tasks necessary in
* a paced-loop structure.
* If more tasks were implemented in the main loop, the delay
* would be adjusted (or eliminated) to provide the desired
* scroll rate.
*-------------------------------------------------------------
MainLoop   JSR   UpdateLCD  ;Update the LCD display
           LDA   #!250
           JSR   Delay      ;Wait 250ms
           BRA   MainLoop   ;Repeat

*-------------------------------------------------------------
* SUBROUTINES
*-------------------------------------------------------------
*-------------------------------------------------------------
* Initialize the message variables for the desired output string
* Register A contains the offset of desired message.
*-------------------------------------------------------------
LoadMsg    STA   MsgIndex   ;Setup the message index
           STA   MsgStart   ;Store the start of the message
           RTS              ;Return

*-------------------------------------------------------------
* Update the LCD with current portion of string to be displayed
*-------------------------------------------------------------
UpdateLCD  LDX   MsgIndex   ;Start at current index into message
           JSR   ShowString ;Show current portion of string
           INC   MsgIndex   ;Increment the index
           RTS              ;Return

*-------------------------------------------------------------
* Show the current string portion on the display.
* When called, the X register contains the index offset.
*-------------------------------------------------------------
ShowString LDA   #LCDDR     ;First LCD data register
           STA   LCDReg     ;LCDReg = First LCD data register
           CLR   Count      ;Clear the counter variable
NextByte   LDA   Msgs,X     ;Load ASCII byte of string
           CMP   #EOT       ;Check for end of string
           BEQ   Padding    ;Last character reached
           JSR   ShowChar   ;Display character
           INCX             ;Increment the index
           INC   Count      ;Increment the counter
           LDA   Count      ;Check the counter
           CMP   #MAXCHARS  ;for LCD display length
           BEQ   Done       ;End of display line reached
           BRA   NextByte   ;Ready the next byte
Padding    LDA   Count      ;Pad the rest of the display with spaces
           CMP   #$00       ;See if string has scrolled off display
           BEQ   Reset      ;Need to reset string
           CMP   #MAXCHARS  ;Check for end of display
           BEQ   Done       ;Finished displaying padding spaces
           INC   Count      ;Increment counter
           JSR   BlankSpace ;Put space in current display position
           BRA   Padding    ;Repeat
Reset      JSR   BlankSpace ;Show a final space in first position
           LDX   MsgStart   ;Load start of message index
           DECX             ;Compensate for INCX in UpdateLCD after RTS
           STX   MsgIndex   ;Record new message index
Done       RTS              ;Return

*-------------------------------------------------------------
* ShowChar converts an ASCII character value in Register A to an
* offset into the character table. The two bytes at the offset
* location of the table define the segment values for displaying
* the character on the display. Then use the offset into the LCD
* data table to get the 2 bytes for the LCD position, and store
* them in the appropriate LCD data registers.
*-------------------------------------------------------------
ShowChar   STX   TempX      ;Save X register
           JSR   Convert    ;Convert ASCII byte into table offset
           TAX              ;Put offset into X
           LDA   Table+1,X  ;Get second LCD data byte
           STA   TempData   ;Store it temporarily
           LDA   Table,X    ;Load A with first LCD data byte
           LDX   LCDReg     ;Point X to current LCD data register
           STA   0,X        ;Store first byte to LCD data register
           LDA   TempData   ;Load A with second data byte
           STA   1,X        ;Store it to second LCD data register
           INC   LCDreg     ;Increment LCDreg pointer to
           INC   LCDReg     ;point to the next position's regs.
           LDX   TempX      ;Restore X register
           RTS              ;Return

*-------------------------------------------------------------
* Convert ASCII character byte in A to an offset value into
* the table of LCD segment values.
* The software also checks for an invalid or unusable ASCII
* character value, and shows a blank space in its place.
* Valid ASCII values are (decimal): 32-47, 48-57, 65-90
*-------------------------------------------------------------
Convert    CMP   #!48       ;Check for "special" character
           BLO   Special
           CMP   #!65       ;Check for numeric character
           BLO   Numeric
Alpha      CMP   #!90       ;Check for invalid value
           BHI   ConvError
           SUB   #!39       ;Convert to table offset
           BRA   ConvDone
Special    CMP   #!32       ;Check for invalid value
           BLO   ConvError
           SUB   #!32       ;Convert to table offset
           BRA   ConvDone
Numeric    CMP   #!57       ;Check for invalid value
           BHI   ConvError
           SUB   #!32       ;Convert to table offset
           BRA   ConvDone
ConvError  CLRA             ;Invalid value shows as blank space
ConvDone   ROLA             ;Multiply offset by 2
           RTS              ;(2 bytes data per LCD position)

*-------------------------------------------------------------
* BlankSpace shows a space ($0000) at the current display position's
* LCD data registers.
*-------------------------------------------------------------
BlankSpace LDX   LCDReg     ;Point to current LCD data register
           CLR   0,X        ;Clear first data byte
           CLR   1,X        ;Clear second data byte
           INC   LCDReg     ;Increment LCDreg pointer to
           INC   LCDReg     ;point to the next position's regs.
           RTS              ;Return

*-------------------------------------------------------------
* Delay for ~Accumulator*1ms (fop = 1MHz)
* A contains the number of 1ms delays desired
*-------------------------------------------------------------
Delay      CMP   #$00       ;Check for remaining delays
           BEQ   DDone      ;Done?
           BSR   MsDelay    ;Call 1ms delay routine
           DECA             ;Decrement count
           BRA   Delay      ;Repeat
DDone      RTS              ;Return

*-------------------------------------------------------------
* Delay for ~1ms (fop = 1MHz)
*-------------------------------------------------------------
MsDelay    STA   TempA
           LDA   #$5A
MsLoop     CMP   #$00
           BEQ   MsDone
           DECA
           BRA   MsLoop
MsDone     LDA    TEMPA
           RTS

*-------------------------------------------------------------
* Interrupt service routine
* This allows a switch on IRQ1 to switch between message strings
*-------------------------------------------------------------
ISR        LDA   MsgStart   ;Start of current message
           CMP   #Msg1      ;Determine next message
           BEQ   Load2
           CMP   #Msg2
           BEQ   Load3
Load1      LDA   #Msg1      ;Load message 1
           BRA   Load
Load2      LDA   #Msg2      ;Load message 2
           BRA   Load
Load3      LDA   #Msg3      ;Load message 3
Load       JSR   LoadMsg
           BSET  RIRQ1,INTSR ;Clear IRQ1 flag
           RTI               ;Return

*-------------------------------------------------------------
* ROM Constants
*-------------------------------------------------------------
*-------------------------------------------------------------
* "Canned" messages
* Each individual message is identified by its offset into the
* base address labelled Msgs
* This limits user to 8 bits of offset (255 characters worth)
* If more than 255 characters are desired for messages, one can
* use some 2-byte variable which can contain multiple base addresses
*
* Valid characters are 0-9, A-Z (UPPERCASE ONLY!), and certain
* special characters defined in the table as valid.
*-------------------------------------------------------------
Msgs       EQU   *          ;Base address of messages
;-------------------------------------------------------------
Msg1       EQU   *-Msgs     ;First message offset
           FCB   "** MOTOROLA MICROCONTROLLERS **"
           FCB   EOT        ;End of text (EOT) marker
;-------------------------------------------------------------
Msg2       EQU   *-Msgs     ;Second message offset
           FCB   "SCROLLING MESSAGE DEMONSTRATION"
           FCB   EOT        ;End of text
;-------------------------------------------------------------
Msg3       EQU   *-Msgs     ;Third message offset
           FCB   "705L16 LCD INTERFACE"
           FCB   EOT        ;End of text
;-------------------------------------------------------------
EndMsgs    EQU   *-Msgs     ;End of messages label


*-------------------------------------------------------------
* Lookup table of LCD segment values for ASCII character values
* Some characters can not be displayed on 15-segment LCD, so
* they are marked as invalid, and will be displayed as a blank
* space.
*-------------------------------------------------------------
Table      FDB   $0000     ;' '
           FDB   $0000     ;'!' INVALID
           FDB   $0201     ;'"'
           FDB   $0000     ;'#' INVALID
           FDB   $A5A5     ;'$'
           FDB   $0000     ;'%' INVALID
           FDB   $0000     ;'&' INVALID
           FDB   $0001     ;'''
           FDB   $000A     ;'('
           FDB   $5000     ;')'
           FDB   $F00F     ;'*'
           FDB   $A005     ;'+'
           FDB   $0000     ;',' INVALID
           FDB   $2004     ;'-'
           FDB   $0800     ;'.'
           FDB   $4002     ;'/'
           FDB   $47E2     ;'0'
           FDB   $0602     ;'1'
           FDB   $23C4     ;'2'
           FDB   $2784     ;'3'
           FDB   $2624     ;'4'
           FDB   $21A8     ;'5'
           FDB   $25E4     ;'6'
           FDB   $0700     ;'7'
           FDB   $27E4     ;'8'
           FDB   $27A4     ;'9'
           FDB   $2764     ;'A'
           FDB   $8785     ;'B'
           FDB   $01E0     ;'C'
           FDB   $8781     ;'D'
           FDB   $21E4     ;'E'
           FDB   $2164     ;'F'
           FDB   $05E4     ;'G'
           FDB   $2664     ;'H'
           FDB   $8181     ;'I'
           FDB   $06C0     ;'J'
           FDB   $206A     ;'K'
           FDB   $00E0     ;'L'
           FDB   $1662     ;'M'
           FDB   $1668     ;'N'
           FDB   $07E0     ;'O'
           FDB   $2364     ;'P'
           FDB   $07E8     ;'Q'
           FDB   $236C     ;'R'
           FDB   $25A4     ;'S'
           FDB   $8101     ;'T'
           FDB   $06E0     ;'U'
           FDB   $4062     ;'V'
           FDB   $4668     ;'W'
           FDB   $500A     ;'X'
           FDB   $9002     ;'Y'
           FDB   $4182     ;'Z'
EndTable   EQU   *-Table   ;End of table label

*-------------------------------------------------------------
* Vector definitions
*-------------------------------------------------------------
          ORG    RESETVEC     ;Reset vector
          FDB    Start
          ORG    IRQVEC       ;IRQ vector
          FDB    ISR


